precision mediump float;

uniform sampler2D 	in_texture;
uniform sampler2D 	in_texture2;
uniform vec2		in_pixel_atlas_size;
uniform vec2 		in_palette_atlas_size;
varying vec3 		texcoord;

uniform int			debugcolor;

#if defined(USE_COLORMAP)
	uniform sampler2D 	in_texture_colormap;
#endif

#if defined(TERRAIN)
	uniform vec2 	in_map_size;	// For colormap texcoord calculation
	varying vec2 	vpos_xz;	// is really vpos xz (used for colormap)
#endif

#if defined(SPRITE)
	varying vec4 	vcolor;
	uniform vec4	in_color;	// NOTE: this is NOT attribute in_vcolor, this is one color for one drawing sprite!
#endif

#if defined(SPRITE2D)
	//varying vec4 	vcolor;
	uniform vec4	in_color;	// NOTE: this is NOT attribute in_vcolor, this is one color for one drawing sprite!
#endif


vec4 val2color(int val)
{
	val = val - 10 * ( val / 10 );
	
//	int val2 = val / 10;
//	val = val - val2 * 10;	// val = val%7;
	
	if ( val == 0 ) return vec4(1.0, 1.0, 1.0, 1.0);
	else if ( val == 1 ) return vec4(0.5, 1.0, 1.0, 1.0);
	else if ( val == 2 ) return vec4(0.0, 1.0, 1.0, 1.0);
	else if ( val == 3 ) return vec4(1.0, 0.5, 1.0, 1.0);
	else if ( val == 4 ) return vec4(1.0, 0.0, 1.0, 1.0);
	else if ( val == 5 ) return vec4(1.0, 1.0, 0.5, 1.0);
	else if ( val == 6 ) return vec4(1.0, 1.0, 0.0, 1.0);
	else if ( val == 7 ) return vec4(0.0, 0.0, 0.0, 1.0);
	else if ( val == 8 ) return vec4(1.0, 0.0, 0.0, 1.0);
	else if ( val == 9 ) return vec4(0.0, 1.0, 0.0, 1.0);
	else return vec4(0.0, 0.0, 0.5, 1.0);
}


int mymod(int x, int y)
{
	return x - y * ( x / y );
}

void main()
{
	vec2 color_xy = vec2(texcoord.x/in_pixel_atlas_size.x, texcoord.y/in_pixel_atlas_size.y);
	vec4 color_idx_vec4 = texture2D(in_texture,color_xy);
	
	int side = 16;
	int color_idx = int(color_idx_vec4.a * 255.0 + 0.5);

	int pal_atlas_size_x = int(in_palette_atlas_size.x);	// TODO: change to int uniforms
	int pal_atlas_size_y = int(in_palette_atlas_size.y);
	
	int max_palettes_x = pal_atlas_size_x/side;
	int max_palettes_y = pal_atlas_size_y/side;
		
	// texcoord.z indicates palette slot
	int slot_i = int(texcoord.z+0.5);
	int slot_y = slot_i / max_palettes_x;
	int slot_x = slot_i - slot_y * max_palettes_x;

	int pal_x = slot_x * side;
	int pal_y = slot_y * side;
	
	int color_y = color_idx / side;
	int color_x = color_idx - color_y * side;
	
	color_x = color_x + pal_x;
	color_y = color_y + pal_y;

	//int global_color_i = color_x + color_y * pal_atlas_size_x;
	

	//gl_FragColor = val2color(debugcolor);
	//return;

	//float c = float(global_color_i) / float(pal_atlas_size_x*pal_atlas_size_y);
	//gl_FragColor = vec4(c,c,c, 1.0);
	//return;
	
	//int global_color_x = mymod(global_color_i, pal_atlas_size_x);
	//int global_color_y = global_color_i / pal_atlas_size_x;

	vec2 coord_in_pal_atlas = vec2( float(color_x) / in_palette_atlas_size.x, 
									float(color_y) / in_palette_atlas_size.y);
	
	vec4 color = texture2D(in_texture2,coord_in_pal_atlas);
	
//	float global_color_y = floor(global_color_idx / in_palette_atlas_size.x);
//	float global_color_x = global_color_idx - global_color_y * in_palette_atlas_size.x;
//	vec2 coord_in_pal_atlas =  vec2 ( global_color_x / in_palette_atlas_size.x, global_color_y / in_palette_atlas_size.y );	
//	
//
//	
//	vec4 color = texture2D(in_texture2,coord_in_pal_atlas);
//	
#if !defined(WATER) && !defined(TERRAIN)
	if ( (color.r > 0.9 && color.g < 0.1 && color.b > 0.9) || color.a < 0.1 ) 
		discard;
#endif
	
#if defined(TERRAIN) && defined(USE_COLORMAP)
	vec2 texcoord_colormap;
//	texcoord_colormap.x = -( (vpos_xz.x) / 10.0 + in_map_size.x / 2.0 ) / in_map_size.x;
//	texcoord_colormap.y =  ( (vpos_xz.y) / 10.0 + in_map_size.y / 2.0 ) / in_map_size.y;

	texcoord_colormap.x = ( in_map_size.x / 2.0 - vpos_xz.x / 10.0 ) / in_map_size.x;
	texcoord_colormap.y = ( in_map_size.y / 2.0 + vpos_xz.y / 10.0 ) / in_map_size.y;

	vec4 color2 = texture2D(in_texture_colormap,texcoord_colormap);
	color.x = color.x * color2.x;
	color.y = color.y * color2.y;
	color.z = color.z * color2.z;
#endif


#if defined(WATER)
	color.a = 0.5;
#endif

#if defined(SPRITE)
	color = color * vcolor * in_color;
#endif

#if defined(SPRITE2D)
	color = color * in_color;
#endif

#if defined(DEBUG3D)
	color = vec4(1.0, 0.0, 0.0, 1.0);
#endif

	gl_FragColor = color;

//	debug
//	gl_FragColor = vec4(1.0, 0.0, 0.0, 1.0);
}
