attribute vec4 	in_vpos;
attribute vec3 	in_texcoord;
uniform mat4 	in_wvp_mat;
varying vec3 texcoord;

#if defined(TERRAIN)
	varying vec2 vpos_xz;
#endif

#if defined(OBJECT)
	attribute float in_vmeshid;	// Is really array of ints, but GLSL doesn't allow int attributes
	uniform mat4 	in_obj_mesh_mats[50];	// TODO: change to c_ro_rsm.MAX_MESHES
#endif

#if defined(WATER)
	uniform float 	in_frame_idx;	// Is really int, but some devices don't allow ints
	uniform vec3 	in_frame_texcoords[32];	// TODO: change to maxframes
#endif

#if defined(SPRITE)
	attribute vec4 	in_vcolor;
	uniform mat4 	in_transformation_mat;
	uniform vec3	in_vcamera;
	uniform float	in_camera_dist;
	varying vec4 	vcolor;
#endif

#if defined(SIMPLE3DOBJECT)
	uniform mat4 	in_transformation_mat;
#endif

#if defined(SPRITE2D)
	uniform mat4 	in_transformation_mat;
	//attribute vec4 	in_vcolor;
	//varying vec4 	vcolor;
#endif


void main()
{
#if defined(TERRAIN)
	//gl_Position	= in_wvp_mat * vec4(in_vpos.x, in_vpos.y/10.0, in_vpos.z, in_vpos.w);	// For ShortBuffer
	gl_Position	= in_wvp_mat * in_vpos;
	texcoord 	= in_texcoord;
	vpos_xz		= in_vpos.xz;
#endif

#if defined(OBJECT)
	gl_Position	= in_wvp_mat * in_obj_mesh_mats[int(in_vmeshid)] * in_vpos;
	texcoord	= in_texcoord;
#endif

#if defined(SPRITE)
	// We decrease sprite Z after projecting to make sprites a little closer to camera,
	// to avoid bugs appearing when some 3d object is behind a sprite 
	vec4 vpos = in_wvp_mat * in_transformation_mat * in_vpos;
	float rotx = 3.14 / 4.0;
	float zdelta_add = 55.0;
	float zdelta_mul = 2.5;
	float delta = (zdelta_add + zdelta_mul * rotx * in_vpos.y) / in_camera_dist;
	vpos.z -= delta;
	gl_Position = vpos;

	texcoord	= in_texcoord;
	vcolor		= in_vcolor/255.0;	// We use unsigned byte quads in range 0-255 for color
#endif

#if defined(WATER)
	gl_Position	= in_wvp_mat * in_vpos;
	texcoord.x	= in_texcoord.x - in_frame_texcoords[0].x + in_frame_texcoords[int(in_frame_idx)].x;
	texcoord.y	= in_texcoord.y - in_frame_texcoords[0].y + in_frame_texcoords[int(in_frame_idx)].y;
	texcoord.z	= in_frame_texcoords[int(in_frame_idx)].z;
#endif

#if defined(DEBUG3D)
	gl_Position	= in_wvp_mat * in_vpos;
	texcoord = in_texcoord;
#endif

#if defined(SPRITE2D)
	gl_Position = in_transformation_mat * vec4(in_vpos.x, in_vpos.y, 0.5, 1.0);
	//vcolor		= in_vcolor/255.0;	// We use unsigned byte quads in range 0-255 for color
	texcoord = in_texcoord;
#endif

#if defined(SIMPLE3DOBJECT)
	gl_Position = in_wvp_mat * in_transformation_mat * in_vpos;
	texcoord	= in_texcoord;
#endif
}	
 